@@ -17,10 +17,11 @@
 """Core compression logic for compressing streams of related files."""
 
 import time
-from typing import Type
 import zlib
+from typing import Type
 
 from ..lazy_import import lazy_import
+
 lazy_import(globals(), """
 from breezy import (
     debug,
@@ -36,23 +37,14 @@ from breezy.bzr import (
 from breezy.i18n import gettext
 """)
 
-from .. import (
-    errors,
-    osutils,
-    trace,
-)
-from .btree_index import BTreeBuilder
+from .. import errors, osutils, trace
 from ..lru_cache import LRUSizeCache
-from .versionedfile import (
-    _KeyRefs,
-    adapter_registry,
-    AbsentContentFactory,
-    ChunkedContentFactory,
-    ExistingContent,
-    FulltextContentFactory,
-    VersionedFilesWithFallbacks,
-    UnavailableRepresentation,
-)
+from .btree_index import BTreeBuilder
+from .versionedfile import (AbsentContentFactory, ChunkedContentFactory,
+                            ExistingContent, FulltextContentFactory,
+                            UnavailableRepresentation,
+                            VersionedFilesWithFallbacks, _KeyRefs,
+                            adapter_registry)
 
 # Minimum number of uncompressed bytes to try fetch at once when retrieving
 # groupcompress blocks.
@@ -95,7 +87,7 @@ class DecompressCorruption(errors.BzrError):
 
     def __init__(self, orig_error=None):
         if orig_error is not None:
-            self.orig_error = ", %s" % (orig_error,)
+            self.orig_error = ", {}".format(orig_error)
         else:
             self.orig_error = ""
         errors.BzrError.__init__(self)
@@ -107,7 +99,7 @@ class DecompressCorruption(errors.BzrError):
 _ZLIB_DECOMP_WINDOW = 32 * 1024
 
 
-class GroupCompressBlock(object):
+class GroupCompressBlock:
     """An object which maintains the internal structure of the compressed data.
 
     This tracks the meta info (start of text, length, type, etc.)
@@ -257,7 +249,7 @@ class GroupCompressBlock(object):
         elif header == cls.GCB_LZ_HEADER:
             out._compressor_name = 'lzma'
         else:
-            raise ValueError('unknown compressor: %r' % (header,))
+            raise ValueError('unknown compressor: {!r}'.format(header))
         out._parse_bytes(bytes, 6)
         return out
 
@@ -364,7 +356,7 @@ class GroupCompressBlock(object):
             kind = self._content[pos:pos + 1]
             pos += 1
             if kind not in (b'f', b'd'):
-                raise ValueError('invalid kind character: %r' % (kind,))
+                raise ValueError('invalid kind character: {!r}'.format(kind))
             content_len, len_len = decode_base128_int(
                 self._content[pos:pos + 5])
             pos += len_len
@@ -416,7 +408,7 @@ class GroupCompressBlock(object):
         return result
 
 
-class _LazyGroupCompressFactory(object):
+class _LazyGroupCompressFactory:
     """Yield content from a GroupCompressBlock on demand."""
 
     def __init__(self, key, parents, manager, start, end, first):
@@ -450,7 +442,7 @@ class _LazyGroupCompressFactory(object):
         self._end = end
 
     def __repr__(self):
-        return '%s(%s, first=%s)' % (self.__class__.__name__,
+        return '{}({}, first={})'.format(self.__class__.__name__,
                                      self.key, self._first)
 
     def _extract_bytes(self):
@@ -497,7 +489,7 @@ class _LazyGroupCompressFactory(object):
                                         self.storage_kind)
 
 
-class _LazyGroupContentManager(object):
+class _LazyGroupContentManager:
     """This manages a group of _LazyGroupCompressFactory objects."""
 
     _max_cut_fraction = 0.75  # We allow a block to be trimmed to 75% of
@@ -713,7 +705,7 @@ class _LazyGroupContentManager(object):
         elif action == 'rebuild':
             self._rebuild_block()
         else:
-            raise ValueError('unknown rebuild action: %r' % (action,))
+            raise ValueError('unknown rebuild action: {!r}'.format(action))
 
     def _wire_bytes(self):
         """Return a byte stream suitable for transmitting over the wire."""
@@ -772,7 +764,7 @@ class _LazyGroupContentManager(object):
          block_len, rest) = bytes.split(b'\n', 4)
         del bytes
         if storage_kind != b'groupcompress-block':
-            raise ValueError('Unknown storage kind: %s' % (storage_kind,))
+            raise ValueError('Unknown storage kind: {}'.format(storage_kind))
         z_header_len = int(z_header_len)
         if len(rest) < z_header_len:
             raise ValueError('Compressed header len shorter than all bytes')
@@ -818,12 +810,12 @@ class _LazyGroupContentManager(object):
 
 def network_block_to_records(storage_kind, bytes, line_end):
     if storage_kind != 'groupcompress-block':
-        raise ValueError('Unknown storage kind: %s' % (storage_kind,))
+        raise ValueError('Unknown storage kind: {}'.format(storage_kind))
     manager = _LazyGroupContentManager.from_bytes(bytes)
     return manager.get_record_stream()
 
 
-class _CommonGroupCompressor(object):
+class _CommonGroupCompressor:
 
     def __init__(self, settings=None):
         """Create a GroupCompressor."""
@@ -968,7 +960,7 @@ class PythonGroupCompressor(_CommonGroupCompressor):
 
         Used only if the pyrex version is not available.
         """
-        super(PythonGroupCompressor, self).__init__(settings)
+        super().__init__(settings)
         self._delta_index = LinesDeltaIndex([])
         # The actual content is managed by LinesDeltaIndex
         self.chunks = self._delta_index.lines
@@ -1023,7 +1015,7 @@ class PyrexGroupCompressor(_CommonGroupCompressor):
     """
 
     def __init__(self, settings=None):
-        super(PyrexGroupCompressor, self).__init__(settings)
+        super().__init__(settings)
         max_bytes_to_index = self._settings.get('max_bytes_to_index', 0)
         self._delta_index = DeltaIndex(max_bytes_to_index=max_bytes_to_index)
 
@@ -1122,7 +1114,7 @@ def cleanup_pack_group(versioned_files):
     versioned_files.stream.close()
 
 
-class _BatchingBlockFetcher(object):
+class _BatchingBlockFetcher:
     """Fetch group compress blocks in batches.
 
     :ivar total_bytes: int of expected number of bytes needed to fetch the
@@ -1176,8 +1168,7 @@ class _BatchingBlockFetcher(object):
 
     def _flush_manager(self):
         if self.manager is not None:
-            for factory in self.manager.get_record_stream():
-                yield factory
+            yield from self.manager.get_record_stream()
             self.manager = None
             self.last_read_memo = None
 
@@ -1203,8 +1194,7 @@ class _BatchingBlockFetcher(object):
                 # We are starting a new block. If we have a
                 # manager, we have found everything that fits for
                 # now, so yield records
-                for factory in self._flush_manager():
-                    yield factory
+                yield from self._flush_manager()
                 # Now start a new manager.
                 if memos_to_get_stack and memos_to_get_stack[-1] == read_memo:
                     # The next block from _get_blocks will be the block we
@@ -1224,8 +1214,7 @@ class _BatchingBlockFetcher(object):
             start, end = index_memo[3:5]
             self.manager.add_factory(key, parents, start, end)
         if full_flush:
-            for factory in self._flush_manager():
-                yield factory
+            yield from self._flush_manager()
         del self.keys[:]
         self.batch_memos.clear()
         del self.memos_to_get[:]
@@ -1627,8 +1616,8 @@ class GroupCompressVersionedFiles(VersionedFilesWithFallbacks):
             # would be better to not globally sort initially but instead
             # start with one key, recurse to its oldest parent, then grab
             # everything in the same group, etc.
-            parent_map = dict((key, details[2]) for key, details in
-                              locations.items())
+            parent_map = {key: details[2] for key, details in
+                              locations.items()}
             for key in unadded_keys:
                 parent_map[key] = self._unadded_refs[key]
             parent_map.update(fallback_parent_map)
@@ -1656,24 +1645,19 @@ class GroupCompressVersionedFiles(VersionedFilesWithFallbacks):
                     if key in self._unadded_refs:
                         # Flush batch, then yield unadded ref from
                         # self._compressor.
-                        for factory in batcher.yield_factories(full_flush=True):
-                            yield factory
+                        yield from batcher.yield_factories(full_flush=True)
                         chunks, sha1 = self._compressor.extract(key)
                         parents = self._unadded_refs[key]
                         yield ChunkedContentFactory(key, parents, sha1, chunks)
                         continue
                     if batcher.add_key(key) > BATCH_SIZE:
                         # Ok, this batch is big enough.  Yield some results.
-                        for factory in batcher.yield_factories():
-                            yield factory
+                        yield from batcher.yield_factories()
             else:
-                for factory in batcher.yield_factories(full_flush=True):
-                    yield factory
-                for record in source.get_record_stream(keys, ordering,
-                                                       include_delta_closure):
-                    yield record
-        for factory in batcher.yield_factories(full_flush=True):
-            yield factory
+                yield from batcher.yield_factories(full_flush=True)
+                yield from source.get_record_stream(keys, ordering,
+                                                       include_delta_closure)
+        yield from batcher.yield_factories(full_flush=True)
 
     def get_sha1s(self, keys):
         """See VersionedFiles.get_sha1s()."""
@@ -1948,7 +1932,7 @@ class GroupCompressVersionedFiles(VersionedFilesWithFallbacks):
         return result
 
 
-class _GCBuildDetails(object):
+class _GCBuildDetails:
     """A blob of data about the build details.
 
     This stores the minimal data, which then allows compatibility with the old
@@ -1967,7 +1951,7 @@ class _GCBuildDetails(object):
          self._delta_end) = position_info
 
     def __repr__(self):
-        return '%s(%s, %s)' % (self.__class__.__name__,
+        return '{}({}, {})'.format(self.__class__.__name__,
                                self.index_memo, self._parents)
 
     @property
@@ -1996,7 +1980,7 @@ class _GCBuildDetails(object):
         return 4
 
 
-class _GCGraphIndex(object):
+class _GCGraphIndex:
     """Mapper from GroupCompressVersionedFiles needs into GraphIndex storage."""
 
     def __init__(self, graph_index, is_locked, parents=True,
@@ -2074,7 +2058,7 @@ class _GCGraphIndex(object):
                 node_refs = static_tuple.as_tuples(node_refs)
                 passed = static_tuple.as_tuples(keys[key])
                 if node_refs != passed[1]:
-                    details = '%s %s %s' % (key, (value, node_refs), passed)
+                    details = '{} {} {}'.format(key, (value, node_refs), passed)
                     if self._inconsistency_fatal:
                         raise knit.KnitCorrupt(self, "inconsistent details"
                                                " in add_records: %s" %
@@ -2250,22 +2234,14 @@ class _GCGraphIndex(object):
 GroupCompressor: Type[_CommonGroupCompressor]
 
 
-from ._groupcompress_py import (
-    apply_delta,
-    apply_delta_to_source,
-    encode_base128_int,
-    decode_base128_int,
-    decode_copy_instruction,
-    LinesDeltaIndex,
-    )
+from ._groupcompress_py import (LinesDeltaIndex, apply_delta,
+                                apply_delta_to_source, decode_base128_int,
+                                decode_copy_instruction, encode_base128_int)
+
 try:
-    from ._groupcompress_pyx import (  # type: ignore
-        apply_delta,
-        apply_delta_to_source,
-        DeltaIndex,
-        encode_base128_int,
-        decode_base128_int,
-        )
+    from ._groupcompress_pyx import (DeltaIndex, apply_delta,  # type: ignore
+                                     apply_delta_to_source, decode_base128_int,
+                                     encode_base128_int)
     GroupCompressor = PyrexGroupCompressor
 except ImportError as e:
     osutils.failed_to_load_extension(e)
