@@ -15,36 +15,22 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from io import BytesIO
 import os
+from io import BytesIO
 
-from .. import (
-    conflicts,
-    errors,
-    symbol_versioning,
-    trace,
-    transport,
-    workingtree,
-    )
-from ..bzr import (
-    bzrdir,
-    conflicts as _mod_bzr_conflicts,
-    workingtree as bzrworkingtree,
-    workingtree_3,
-    workingtree_4,
-    )
+from .. import (conflicts, errors, symbol_versioning, trace, transport,
+                workingtree)
+from ..bzr import bzrdir
+from ..bzr import conflicts as _mod_bzr_conflicts
+from ..bzr import workingtree as bzrworkingtree
+from ..bzr import workingtree_3, workingtree_4
 from ..lock import write_locked
 from ..lockdir import LockDir
+from ..tree import TreeDirectory, TreeEntry, TreeFile, TreeLink
 from . import TestCase, TestCaseWithTransport, TestSkipped
-from ..tree import (
-    TreeEntry,
-    TreeDirectory,
-    TreeFile,
-    TreeLink,
-    )
-
 from .features import SymlinkFeature
 
+
 class TestTreeDirectory(TestCaseWithTransport):
 
     def test_kind_character(self):
@@ -297,7 +283,7 @@ class TestWorkingTreeIterEntriesByDir_wSubtrees(TestCaseWithTransport):
 class TestWorkingTreeFormatRegistry(TestCase):
 
     def setUp(self):
-        super(TestWorkingTreeFormatRegistry, self).setUp()
+        super().setUp()
         self.registry = workingtree.WorkingTreeFormatRegistry()
 
     def test_register_unregister_format(self):
@@ -487,7 +473,7 @@ class TestAutoResolve(TestCaseWithTransport):
         remaining, resolved = tree.auto_resolve()
         self.assertEqual(
             remaining,
-            conflicts.ConflictList([_mod_bzr_conflicts.TextConflict(u'hello', 'hello-id')]))
+            conflicts.ConflictList([_mod_bzr_conflicts.TextConflict('hello', 'hello-id')]))
         self.assertEqual(resolved, [])
 
     def test_auto_resolve_missing(self):
@@ -498,7 +484,7 @@ class TestAutoResolve(TestCaseWithTransport):
         self.assertEqual(remaining, [])
         self.assertEqual(
             resolved,
-            conflicts.ConflictList([_mod_bzr_conflicts.TextConflict(u'hello', 'hello-id')]))
+            conflicts.ConflictList([_mod_bzr_conflicts.TextConflict('hello', 'hello-id')]))
 
 
 class TestStoredUncommitted(TestCaseWithTransport):
